/***    simpconn.c - simple connection
*
*      Copyright(C) StarQuest Ventures, Inc., 1999-2014
***/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define true 1
#define false 0

#define MAX_STRING_LENGTH	50
#define MAX_CONNECTS		1000
#define MAX_STRING_SIZE		700
#define MAX_NUM_BUFFER		60
#define MAX_ERROR_SIZE		200

#define OK(rc)				(rc==SQL_SUCCESS||rc==SQL_SUCCESS_WITH_INFO)
#ifdef _WIN32
#include <windows.h>
#endif 
#include <sql.h>
#include <sqlext.h>	
#ifdef UNIX
#include <unistd.h>
#if defined(_OLD_DRIVER_MANAGER_)
#	define SQLHENV HENV
#	define SQLHDBC HDBC
#	define SQLHSTMT HSTMT
#	define SQLCHAR UCHAR
#	define SQLRETURN RETCODE
#endif
#endif



void Usage();
void ConnExceeded();
void ShowDriverInfo(SQLHDBC hdbc);

static time_t t1, t2;

int main(int argc, char* argv[])
{
	SQLHENV henv = SQL_NULL_HENV; 
	SQLHDBC hdbc[MAX_CONNECTS];
	int connected[MAX_CONNECTS];
	SQLRETURN retcode;

	char	szServer[MAX_STRING_LENGTH+1];
	char	szLogin[MAX_STRING_LENGTH+1];
	char	szPassword[MAX_STRING_LENGTH+1];
	char	*stopstring;
	char	szState[6];
	char	buf[MAX_ERROR_SIZE];
	int		iNumConn=1;
	int		iPause=10000;
	int		iCount, pCount;
	int		poolCount = 1;
	BOOL	wait, pool;

	clock_t start, end;

	wait=false;
	pool=false;

	memset(szServer, '\0', sizeof(szServer));
	memset(szLogin, '\0', sizeof(szLogin));
	memset(szPassword, '\0', sizeof(szPassword));
	if (argc >= 2)
	{
		strcpy(szServer, argv[1]);
		if (argc >2 && argv[2] != 0)
			strcpy(szLogin, argv[2]);
		if (argc >3 && argv[3] != 0)
			strcpy(szPassword, argv[3]);
		if (argc >4 ) 
		{
			iNumConn = strtol( argv[4], &stopstring, 10);  
			if (iNumConn > MAX_CONNECTS) 
			{
				ConnExceeded();
				exit(1);
			}
		}
		if (argc > 5 && (strcmp(argv[5],"-wait")) == 0 )
		{
			wait=true;
		}
		if (argc > 5 && (strcmp(argv[5],"-pool")) == 0 )
		{
			pool=true;
		}
		if (argc > 6 ) 
		{
			poolCount = strtol( argv[6], &stopstring, 10);  
		}
	}
	else
	{
		Usage();
		exit(1);
	}

	// This is necessary on Windows but not unix (unixODBC)
	if (pool) {
		if (!SQL_SUCCEEDED(SQLSetEnvAttr(
			NULL,  // make process level cursor pooling
			SQL_ATTR_CONNECTION_POOLING,
			(SQLPOINTER)SQL_CP_ONE_PER_DRIVER,
			SQL_IS_INTEGER)))
			fprintf(stderr, "SQLSetEnvAttr/SQL_ATTR_CONNECTION_POOLING error\n");
	}

	retcode = SQLAllocEnv(&henv);
	if (! OK(retcode))
	{
		fprintf(stderr,"SQLAllocEnv failed\n");
		exit(1);
	}

	t1 = time(0);
	start = clock();

	// Loop for simultaneous connections
	for(iCount=0; iCount < iNumConn; iCount++)
	{
		retcode = SQLAllocConnect(henv, &hdbc[iCount]);
		if (! OK(retcode))
		{
			fprintf(stderr,"SQLAllocConnect failed\n");
			exit(1);
		}

		// Loop for pooling (connect/disconnect)
		for(pCount=0; pCount < poolCount; pCount++)
		{
			retcode = SQLConnect(hdbc[iCount],(SQLCHAR*)szServer, SQL_NTS, 
				(SQLCHAR*)szLogin, SQL_NTS, (SQLCHAR*)szPassword, SQL_NTS);

			connected[iCount] = (retcode == SQL_SUCCESS || retcode == SQL_SUCCESS_WITH_INFO);
			printf("Connection #%d\n",iCount+1);

			if (retcode == SQL_SUCCESS_WITH_INFO ) 
			{
				SQLError(henv, hdbc[iCount], NULL, (SQLCHAR *)szState, NULL, 
					(SQLCHAR *)buf, MAX_ERROR_SIZE, NULL);
				printf("SQL_SUCCESS_WITH_INFO SQLState = %s, %s\n",szState, buf);
			}
			if ((retcode != SQL_SUCCESS) && (retcode != SQL_SUCCESS_WITH_INFO )) 
			{
				SQLError(henv, hdbc[iCount], NULL, (SQLCHAR *)szState, NULL, 
					(SQLCHAR *)buf, MAX_ERROR_SIZE, NULL);
				fprintf(stderr, "SQLState = %s, %s\n",szState, buf);
			}
			else
			{
				ShowDriverInfo(hdbc[iCount]);
			}				
			if (poolCount > 1)
			{
				retcode = connected[iCount] ? SQLDisconnect(hdbc[iCount]) : SQL_SUCCESS;

			}
		}
	}


	/*
	#ifdef _WINDOWS
	Sleep(iPause);
	#else
	sleep(iPause/1000);
	#endif
	*/
	if (wait)
	{
		char input[80];
		printf("Hit Enter to continue\n");
		gets(input);
	}
	if (poolCount < 2)
	{
		for (iCount = iNumConn - 1; iCount >= 0; iCount--)
		{
			retcode = connected[iCount] ? SQLDisconnect(hdbc[iCount]) : SQL_SUCCESS;
			if (! OK(retcode))
			{
				fprintf(stderr,"SQLDisconnect failed\n");
				exit(1);
			}

			retcode = SQLFreeConnect(hdbc[iCount]);
			if (! OK(retcode))
			{
				fprintf(stderr,"SQLFreeConnect failed\n");
				exit(1);
			}
		}

		retcode = SQLFreeEnv(henv);
		if (! OK(retcode))
		{
			fprintf(stderr,"SQLFreeEnv failed\n");
			exit(1);
		}
	}

	t2 = time(0);
	end = clock();
	fprintf(stderr, "Finished in %d seconds; %ld ticks.\n", (int) (t2-t1), end-start);
	return(retcode);
}

void Usage()
{
	fprintf(stderr,"Usage: simpleconn <DSN> <UID> <PWD> [<# of simultaneous connections>] [-wait] | [-pool] | [-nopool] [<# of pooled connections]\n");
	fprintf(stderr,"Arguments are position-sensitive.\n -wait, -pool and -nopool are mutually exclusive\n");
}

void ConnExceeded()
{
	fprintf(stderr,"Error: Number of connections cannot exceed %i\n", MAX_CONNECTS);
	fprintf(stderr,"Please rebuild simpleconn with a larger value for MAX_CONNECTS\n");
}

void ShowDriverInfo(SQLHDBC hdbc)
{
	RETCODE rc=SQL_SUCCESS;
	char		szNum[MAX_NUM_BUFFER];

	rc = SQLGetInfo(hdbc, SQL_DRIVER_NAME, szNum, sizeof(szNum), NULL);
	printf("\tDriver Name: %s\n", szNum);

	rc = SQLGetInfo(hdbc, SQL_DRIVER_VER, szNum, sizeof(szNum), NULL);
	printf("\tDriver Version: %s\n", szNum);

	rc = SQLGetInfo(hdbc, SQL_DBMS_NAME, szNum, sizeof(szNum), NULL);
	printf("\tDatabase Name: %s\n", szNum);

	rc = SQLGetInfo(hdbc, SQL_DBMS_VER, szNum, sizeof(szNum), NULL);
	printf("\tDatabase Version: %s\n", szNum);

}
